
// See: Documentation.dm for the list of automata rules


var/rule1=2 // any alive cell with fewer than rule1 alive neighbors dies (< rule1)
// rule 2 is not necessary to define
var/rule3=3 // any alive cell with more than rule3 alive neighbors dies (> rule3)
var/rule4=3 // any dead cell with exactly rule4 alive neighbors becomes alive (==rule4)

var/generation=0		// current generation/tick




mob/proc
	CellularAutomata()
		// begin the cellular automation


		while(src)
			if(automationToggle)

				generation++
				winset(src,"generation","text=\"[generation]\"")



				var/TransformingCells[]=new() // array of cells that will be transformed in this generation

				for(var/obj/Cell/Alive/A in world)

					if(A.z==1)
						// should only occur in the playing field

						for(var/obj/Cell/Dead/D_ in oview(1,A))

							if(!(D_ in TransformingCells))

							// loop through dead cells

								var/D_neighbors=0 // # of alive neighbors a dead cell has

								for(var/obj/Cell/Alive/A_ in oview(1,D_))

									D_neighbors++

								if(D_neighbors==rule4)
									// if a dead cell has exactly 3 neighbors, it transforms / becomes alive

									TransformingCells.Add(D_)


						var/neighbors=0 // # of alive neighbors an alive cell has

						for(var/obj/Cell/Alive/A_ in oview(1,A))
							neighbors++


						if(neighbors<rule1)
							// if an alive cell has less than 2 neighbors, it transforms / dies
							TransformingCells.Add(A)

						// if an alive cell has 2 or 3 neighbors, it remains unphased

						if(neighbors>rule3)
							// if an alive cell has more than 3 neighbors, it transforms / dies
							TransformingCells.Add(A)



				for(var/obj/Cell/C in TransformingCells)
					// transform any cells that need to be transformed

					C.Transform()







				// after all of that is done, we begin to check if there are any cells to turn into viruses
				// this is mostly unrelated to the actual Conway's Game of Life.

				for(var/obj/Cell/Virus/V in world)
					if(V.z==1)

					/*
					Viruses are ALIVE cells that have been infected. They have a constant set of laws
					that allows them to live for much longer than normal cells. They require the
					presence of live, noninfected cells in order to reproduce.

					Law 1: Any infected cell with fewer than 1 live, uninfected neighbor dies. (nothing to reproduce/infect)
					Law 2: Any infected cell with more than 1 live, uninfected neighbors lives, and infects random neighbor(s), even if they are dead.
					Law 3: Any infected cell with more than 8 live (surrounded), uninfected neighbors dies. (immune system)
					*/

						var/list/neighbors=list() // list of alive neighbors

						for(var/obj/Cell/Alive/A_ in oview(1,V))
							// loop through every alive neighbor and add it into possible selections
							neighbors.Add(A_)


						if(neighbors.len<1) 				V.Transform() // Law 1

						if(neighbors.len>1)

							// Law 2

							for(var/obj/Cell/A_ in neighbors)
								// loops through all cells in the neighbors list

								if(prob(80))
									// Infected cells have an 80% chance of infecting all nearby alive cells

									for(var/obj/Cell/Dead/D_ in oview(1,A_))
										// have a chance to repopulate dead cells if it infects nearby cells

										if(prob(70)) D_.Transform(1) // infect dead cell

									neighbors.Remove(A_)
									A_.Transform(1) // infect live cell

								else if(neighbors.len==1)
									// if the probablity check fails and there's only 1 neighbor, infect it

									for(var/obj/Cell/Dead/D_ in oview(1,A_))
										if(prob(70)) D_.Transform(1)

									neighbors.Remove(A_)
									A_.Transform(1) // infect live cell


						if(neighbors.len>8) 				V.Transform() // law 3


			else break

			sleep(delayTime)


	Clear()
		for(var/obj/Cell/Alive/A in world)
			if(A.z==1)
				var/obj/Cell/Dead/Aa=new/obj/Cell/Dead
				Aa.loc=locate(A.x,A.y,A.z)

				del(A)

		for(var/obj/Cell/Virus/V in world)
			if(V.z==1)
				var/obj/Cell/Dead/Aa=new/obj/Cell/Dead
				Aa.loc=locate(V.x,V.y,V.z)

				del(V)